function im_reconstr = interp_reconstr_fun(frames, frame_locations, method)
% The interpolation reconstruction function
%
% @Inputs
% frames: a 3D array of input LR frames
% frame_locations: an array indicating locations of LR frames
% method: 'linear', 'cubic', 'nearest'
%
% @Output
% im_reconstr: reconstructed image

    im_reconstr = zeros(3*size(frames(:,:,1))+2);
    numSubFrame = length(frame_locations);
    for k=1:numSubFrame
        frame = frames(:,:,k);
        frame_us = imresize(frame,3, method);   % By default uses bicubic interpolation
        frame_us_pad = padarray(frame_us,[2, 2],'post');
        im_reconstr = im_reconstr + circshift(frame_us_pad,[floor(frame_locations(k)/3), mod(frame_locations(k)-1,3)]);    
    end
    im_reconstr = im_reconstr / numSubFrame;
end