% Author: Kaijun Feng
% Date: March 2020

clc; clear;
addpath('altmany-export_fig-ce3632d\');

%% Read original image
im_name = 'birds_gray512.png';
% im = im2double(imread(im_name));
% im_gray = rgb2gray(im);
im_gray = im2double(imread(im_name));
figure; imshow(im_gray);

%% Pad input if needed
to_pad = 2 - mod(size(im_gray,1),3);
if to_pad > 0
   im_gray = padarray(im_gray, [to_pad, to_pad], 'replicate', 'post'); 
end
figure; imshow(im_gray);

%% Create LR frames
kernel = fspecial('average',3);
% kernel = ones(3);
im_avg = conv2(im_gray, kernel, 'valid');
% figure; imshow(im_avg);
sizeLR = size(im_avg,1) / 3;
frame_stack = double.empty(sizeLR, sizeLR, 0);
for r = 1:3
    for c = 1:3
        frame = im_avg(r:3:end, c:3:end);
        frame_stack = cat(3,frame_stack, frame);
    end
end
figure; imshow(frame_stack(:,:,1)); % Aliasing is evident

%% Add Gaussian noise to frames
global sigma
sigma = 0.03;
noisy_frame_stack = frame_stack + sigma.*randn(size(frame_stack));
figure; imshow(noisy_frame_stack(:,:,1), 'border', 'tight'); title('Noisy frame 1');

%% Use some of the 9 frames
arrFrameNum = [1 3 4 5 7 8];
global numSubFrame
numSubFrame = length(arrFrameNum);
[w,h] = size(frame);
subStack = zeros(w,h,numSubFrame);
noisySubStack = subStack;
for k=1:numSubFrame
    frame = frame_stack(:,:,arrFrameNum(k));
    subStack(:,:,k) = frame;
    noisySubStack(:,:,k) = noisy_frame_stack(:,:,arrFrameNum(k));
end

%% Prepare directory to save results
parent_folder = 'results/birds/';
if ~exist(parent_folder,'dir')
    mkdir(parent_folder)
end
sub_folders = ["admm_tv_aniso", "admm_tv_iso", "nlm"];
full_folders = strings(3,1);
for k=1:3
    full_folders(k) = fullfile(parent_folder,sub_folders(k));
   if ~exist(full_folders(k),'dir')
       mkdir(full_folders(k))
   end
end

%% Simple interpolation, shift and add
method = 'bicubic';
im_reconstruct = interp_reconstr_fun(noisySubStack, arrFrameNum,method);
% Check PSNR
im_reconstr_crop = im_reconstruct(3:end-2,3:end-2);
im_gray_crop = im_gray(3:end-2,3:end-2);
psnr_interpolation = psnr(im_reconstr_crop, im_gray_crop);
% Show result
imgFileName = ['interpolation' num2str(numSubFrame) 'frames-sigma'...
    num2str(sigma) 'psnr' num2str(psnr_interpolation,4) '.png'];
figure; imshow(im_reconstruct); title(['PSNR=' num2str(psnr_interpolation,4)], 'FontSize', 16);
% imwrite(im_reconstr_crop,fullfile(parent_folder, imgFileName));
export_fig(fullfile(parent_folder,imgFileName));

%% ADMM-TV-Anisotropic
lambda = 0.005;
rho = 2;
admm_iter = 20;
tic
[Iadmm_tv_aniso, vRes_aniso, vPsnr_aniso] = admm_tv_fun(im_gray,...
    noisySubStack, arrFrameNum, lambda, rho, admm_iter, false);
toc
% Save the results
save_results(Iadmm_tv_aniso, lambda, rho, vPsnr_aniso, vRes_aniso, false, ...
    full_folders(1));
% % Check a LR frame generated from ADMM-TV result
% frame_stack_post_admm = Afun(Iadmm, arrFrameNum);
% figure; imshow(frame_stack_post_admm(:,:,1));

%% ADMM-TV-isotropic
lambda = 0.005;
rho = 2;
admm_iter = 20;
tic
[Iadmm_tv_iso, vRes_iso, vPsnr_iso] = admm_tv_fun(im_gray, noisySubStack, ...
    arrFrameNum, lambda, rho, admm_iter, true);
toc
save_results(Iadmm_tv_iso, lambda, rho, vPsnr_iso, vRes_iso, false, ...
    full_folders(2));

%% ADMM-NLM
lambda = 0.001;
rho = 0.2;
admm_iter = 20;
tic
[Iadmm_nlm, vRes_nlm, vPsnr_nlm] = admm_nlm_fun(im_gray, noisySubStack, ...
    arrFrameNum, lambda, rho, admm_iter);
toc
save_results(Iadmm_nlm, lambda, rho, vPsnr_nlm, vRes_nlm, true, full_folders(3));

%% Utility
function save_results(img, lambda, rho, vPsnr, vResidual, logy, folder)
    global numSubFrame sigma
    % Check psnr
    psnr_admm_tv_aniso = vPsnr(end);
    % Present result
    font_size = 16;
    marker_size = 14;
    imgTitle = ['\lambda=' num2str(lambda) ' \rho=' num2str(rho) ' PSNR='...
        num2str(psnr_admm_tv_aniso, 4)];
    figure; imshow(img); title(imgTitle, 'FontSize', font_size);
    imgFileName = ['ADMM-TV_aniso' num2str(numSubFrame) 'frames-sigma'...
        num2str(sigma) 'lambda' num2str(lambda) 'rho' num2str(rho)...
        'psnr' num2str(psnr_admm_tv_aniso,4) '.png'];
    % imwrite(Iadmm_tv_aniso,[imgName '.png']);
    % set(gcf,'PaperPosition', [0 0 3.2 3.2]);
    % print(['results/' imgFileName], '-r300', '-dpng');
    export_fig(fullfile(folder, imgFileName));
    % Plot
    figure; yyaxis left; 
    if logy
        semilogy(vResidual,'o', 'MarkerSize', marker_size, 'LineWidth', 1); ylabel('Residual', 'FontSize', font_size);
    else
        plot(vResidual,'o', 'MarkerSize', marker_size, 'LineWidth', 1); ylabel('Residual', 'FontSize', font_size);
    end
    yyaxis right; plot(vPsnr,'x', 'MarkerSize', marker_size, 'LineWidth', 1); ylabel('PSNR (dB)', 'FontSize', font_size);
    xlabel('Iterations', 'FontSize', font_size);
    plotTitle = ['\lambda=' num2str(lambda) ' \rho=' num2str(rho)];
    title(plotTitle, 'FontSize', font_size);
    set(gca, 'FontSize', font_size);
    % set(gcf,'PaperPosition', [0, 0, 3.2, 2.4]);
    plotFileName = ['ADMM-TV_aniso' num2str(numSubFrame) 'frames-sigma'...
        num2str(sigma) 'lambda' num2str(lambda) 'rho' num2str(rho) '.png'];
    % print(['results/' plotFileName], '-dpng', '-r300');
    export_fig(fullfile(folder, plotFileName));
end
